// Copyright 2024 Bytedance Inc. All Rights Reserved.
// Author: tianlei.richard@qq.com (tianlei.richard)

#include "triangle.h"
#include <algorithm>
#include <iostream>

Triangle::Triangle(const Point3d &a, const Point3d &b, const Point3d &c)
    : points_({a, b, c}) {}

Triangle::Triangle(const HomoPointType &a, const HomoPointType &b,
                   const HomoPointType &c)
    : points_({(a / a.z()).head(3), (b / b.z()).head(3), (c / c.z()).head(3)}) {
}

BBox Triangle::axis_align_bbox() const {
  const int x_min = std::min({points_[0].x(), points_[1].x(), points_[2].x()});
  const int y_min = std::min({points_[0].y(), points_[1].y(), points_[2].y()});
  const int x_max = std::max({points_[0].x(), points_[1].x(), points_[2].x()});
  const int y_max = std::max({points_[0].y(), points_[1].y(), points_[2].y()});
  return BBox{x_min, y_min, (x_max - x_min + 1), (y_max - y_min + 1)};
}

Vector3d Triangle::normal_vector() const {
  const auto v1 = points_[1] - points_[0];
  const auto v2 = points_[2] - points_[1];
  return v1.cross(v2).normalized();
}

void Triangle::affine_transform(const TransformMatrix &m) {
  for (auto &p : points_) {
    p = (m * Eigen::Vector4d(p.x(), p.y(), p.z(), 1.)).head<3>();
  }
}

void Triangle::projective_transform(const TransformMatrix &m) {
  for (auto &p : points_) {
    auto p_homo = m * Eigen::Vector4d(p.x(), p.y(), p.z(), 1.);
    p = (p_homo / p_homo.w()).head<3>();
  }
}
